@@ -20,7 +20,7 @@
     slouken@libsdl.org
 */
 
-/* $Id: mixer.c,v 1.50 2004/08/21 12:24:02 slouken Exp $ */
+/* $Id: mixer.c 2290 2006-05-01 03:34:46Z slouken $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -275,8 +275,13 @@ int Mix_OpenAudio(int frequency, Uint16 format, int nchannels, int chunksize)
 
 	/* If the mixer is already opened, increment open count */
 	if ( audio_opened ) {
-	    ++audio_opened;
-	    return(0);
+		if ( format == mixer.format && nchannels == mixer.channels ) {
+	    	++audio_opened;
+	    	return(0);
+		}
+		while ( audio_opened ) {
+			Mix_CloseAudio();
+		}
 	}
 
 	/* Set the desired format and frequency */
