@@ -1,6 +1,6 @@
 /*
     SDL_mixer:  An audio mixer library based on the SDL library
-    Copyright (C) 1997, 1998, 1999, 2000, 2001  Sam Lantinga
+    Copyright (C) 1997-2004 Sam Lantinga
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -20,17 +20,26 @@
     slouken@libsdl.org
 */
 
-/* $Id: music.c,v 1.51 2003/02/12 05:07:12 slouken Exp $ */
+/* $Id: music.c 2408 2006-05-12 09:42:24Z slouken $ */
 
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
+#include <assert.h>
 #include "SDL_endian.h"
 #include "SDL_audio.h"
 #include "SDL_timer.h"
 
 #include "SDL_mixer.h"
 
+#define SDL_SURROUND
+
+#ifdef SDL_SURROUND
+#define MAX_OUTPUT_CHANNELS 6
+#else
+#define MAX_OUTPUT_CHANNELS 2
+#endif
+
 /* The music command hack is UNIX specific */
 #ifndef unix
 #undef CMD_MUSIC
@@ -42,12 +51,15 @@
 #ifdef WAV_MUSIC
 #include "wavestream.h"
 #endif
-#ifdef MOD_MUSIC
+#if defined(MOD_MUSIC) || defined(LIBMIKMOD_MUSIC)
 #  include "mikmod.h"
 #  if defined(LIBMIKMOD_VERSION)                /* libmikmod 3.1.8 */
 #    define UNIMOD			MODULE
 #    define MikMod_Init()		MikMod_Init(NULL)
 #    define MikMod_LoadSong(a,b)	Player_Load(a,b,0)
+#    ifndef LIBMIKMOD_MUSIC
+#    define MikMod_LoadSongRW(a,b)	Player_LoadRW(a,b,0)
+#    endif
 #    define MikMod_FreeSong		Player_Free
      extern int MikMod_errno;
 #  else                                        /* old MikMod 3.0.3 */
@@ -72,7 +84,7 @@
 #include "music_ogg.h"
 #endif
 #ifdef MP3_MUSIC
-#include "smpeg.h"
+#include "dynamic_mp3.h"
 
 static SDL_AudioSpec used_mixer;
 #endif
@@ -95,7 +107,7 @@ struct _Mix_Music {
 #ifdef WAV_MUSIC
 		WAVStream *wave;
 #endif
-#ifdef MOD_MUSIC
+#if defined(MOD_MUSIC) || defined(LIBMIKMOD_MUSIC)
 		UNIMOD *module;
 #endif
 #ifdef MID_MUSIC
@@ -128,10 +140,15 @@ static int native_midi_ok;
 #endif
 #endif
 
+/* Reference for converting mikmod output to 4/6 channels */
+static int current_output_channels;
+static Uint16 current_output_format;
+
 /* Used to calculate fading steps */
 static int ms_per_step;
 
 /* Local low-level functions prototypes */
+static void music_internal_initialize_volume(void);
 static void music_internal_volume(int volume);
 static int  music_internal_play(Mix_Music *music, double position);
 static int  music_internal_position(double position);
@@ -150,6 +167,36 @@ void Mix_HookMusicFinished(void (*music_finished)(void))
 }
 
 
+/* If music isn't playing, halt it if no looping is required, restart it */
+/* otherwhise. NOP if the music is playing */
+static int music_halt_or_loop (void)
+{
+	/* Restart music if it has to loop */
+	
+	if (!music_internal_playing()) 
+	{
+		/* Restart music if it has to loop at a high level */
+		if (music_loops && --music_loops)
+		{
+			Mix_Fading current_fade = music_playing->fading;
+			music_internal_play(music_playing, 0.0);
+			music_playing->fading = current_fade;
+		} 
+		else 
+		{
+			music_internal_halt();
+			if (music_finished_hook)
+				music_finished_hook();
+			
+			return 0;
+		}
+	}
+	
+	return 1;
+}
+
+
+
 /* Mixing function */
 void music_mixer(void *udata, Uint8 *stream, int len)
 {
@@ -178,21 +225,11 @@ void music_mixer(void *udata, Uint8 *stream, int len)
 				music_playing->fading = MIX_NO_FADING;
 			}
 		}
-		/* Restart music if it has to loop */
-		if ( !music_internal_playing() ) {
-			/* Restart music if it has to loop at a high level */
-			if ( music_loops && --music_loops ) {
-				Mix_Fading current_fade = music_playing->fading;
-				music_internal_play(music_playing, 0.0);
-				music_playing->fading = current_fade;
-			} else {
-				music_internal_halt();
-				if ( music_finished_hook ) {
-					music_finished_hook();
-				}
-				return;
-			}
-		}
+		
+		if (music_halt_or_loop() == 0)
+			return;
+		
+		
 		switch (music_playing->type) {
 #ifdef CMD_MUSIC
 			case MUS_CMD:
@@ -204,9 +241,59 @@ void music_mixer(void *udata, Uint8 *stream, int len)
 				WAVStream_PlaySome(stream, len);
 				break;
 #endif
-#ifdef MOD_MUSIC
+#if defined(MOD_MUSIC) || defined(LIBMIKMOD_MUSIC)
 			case MUS_MOD:
-				VC_WriteBytes((SBYTE *)stream, len);
+				if (current_output_channels > 2) {
+					int small_len = 2 * len / current_output_channels;
+					int i;
+					Uint8 *src, *dst;
+
+					VC_WriteBytes((SBYTE *)stream, small_len);
+					/* and extend to len by copying channels */
+					src = stream + small_len;
+					dst = stream + len;
+
+					switch (current_output_format & 0xFF) {
+						case 8:
+							for ( i=small_len/2; i; --i ) {
+								src -= 2;
+								dst -= current_output_channels;
+								dst[0] = src[0];
+								dst[1] = src[1];
+								dst[2] = src[0];
+								dst[3] = src[1];
+								if (current_output_channels == 6) {
+									dst[4] = src[0];
+									dst[5] = src[1];
+								}
+							}
+							break;
+						case 16:
+							for ( i=small_len/4; i; --i ) {
+								src -= 4;
+								dst -= 2 * current_output_channels;
+								dst[0] = src[0];
+								dst[1] = src[1];
+								dst[2] = src[2];
+								dst[3] = src[3];
+								dst[4] = src[0];
+								dst[5] = src[1];
+								dst[6] = src[2];
+								dst[7] = src[3];
+								if (current_output_channels == 6) {
+									dst[8] = src[0];
+									dst[9] = src[1];
+									dst[10] = src[2];
+									dst[11] = src[3];
+								}
+							}
+							break;
+					}
+
+
+
+				}
+				else VC_WriteBytes((SBYTE *)stream, len);
 				if ( music_swap8 ) {
 					Uint8 *dst;
 					int i;
@@ -242,12 +329,16 @@ void music_mixer(void *udata, Uint8 *stream, int len)
 #endif
 #ifdef OGG_MUSIC
 			case MUS_OGG:
-				OGG_playAudio(music_playing->data.ogg, stream, len);
+				
+				len = OGG_playAudio(music_playing->data.ogg, stream, len);
+				if (len > 0 && music_halt_or_loop())
+					OGG_playAudio(music_playing->data.ogg, stream, len);
+			
 				break;
 #endif
 #ifdef MP3_MUSIC
 			case MUS_MP3:
-				SMPEG_playAudio(music_playing->data.mp3, stream, len);
+				smpeg.SMPEG_playAudio(music_playing->data.mp3, stream, len);
 				break;
 #endif
 			default:
@@ -261,6 +352,9 @@ void music_mixer(void *udata, Uint8 *stream, int len)
 int open_music(SDL_AudioSpec *mixer)
 {
 	int music_error;
+#ifdef LIBMIKMOD_MUSIC
+	CHAR *list;
+#endif
 
 	music_error = 0;
 #ifdef WAV_MUSIC
@@ -268,7 +362,7 @@ int open_music(SDL_AudioSpec *mixer)
 		++music_error;
 	}
 #endif
-#ifdef MOD_MUSIC
+#if defined(MOD_MUSIC) || defined(LIBMIKMOD_MUSIC)
 	/* Set the MikMod music format */
 	music_swap8 = 0;
 	music_swap16 = 0;
@@ -302,22 +396,37 @@ int open_music(SDL_AudioSpec *mixer)
 			++music_error;
 		}
 	}
+	current_output_channels = mixer->channels;
+	current_output_format = mixer->format;
 	if ( mixer->channels > 1 ) {
-		if ( mixer->channels > 2 ) {
+		if ( mixer->channels > MAX_OUTPUT_CHANNELS ) {
 			Mix_SetError("Hardware uses more channels than mixer");
 			++music_error;
 		}
 		md_mode |= DMODE_STEREO;
 	}
-	md_mixfreq	 = mixer->freq;
-	md_device	  = 0;
-	md_volume	  = 96;
+	md_mixfreq = mixer->freq;
+	md_device  = 0;
+	md_volume  = 96;
 	md_musicvolume = 128;
 	md_sndfxvolume = 128;
-	md_pansep	  = 128;
-	md_reverb	  = 0;
+	md_pansep  = 128;
+	md_reverb  = 0;
+	md_mode    |= DMODE_HQMIXER|DMODE_SOFT_MUSIC|DMODE_SURROUND;
+#ifdef LIBMIKMOD_MUSIC
+	list = MikMod_InfoDriver();
+	if ( list )
+	  free(list);
+	else
+#endif
+	MikMod_RegisterDriver(&drv_nos);
+#ifdef LIBMIKMOD_MUSIC
+	list = MikMod_InfoLoader();
+	if ( list )
+	  free(list);
+	else
+#endif
 	MikMod_RegisterAllLoaders();
-	MikMod_RegisterAllDrivers();
 	if ( MikMod_Init() ) {
 		Mix_SetError("%s", MikMod_strerror(MikMod_errno));
 		++music_error;
@@ -381,7 +490,7 @@ Mix_Music *Mix_LoadMUS(const char *file)
 {
 	FILE *fp;
 	char *ext;
-	Uint8 magic[5];
+	Uint8 magic[5], moremagic[9];
 	Mix_Music *music;
 
 	/* Figure out what kind of file this is */
@@ -393,7 +502,12 @@ Mix_Music *Mix_LoadMUS(const char *file)
 		Mix_SetError("Couldn't read from '%s'", file);
 		return(NULL);
 	}
+	if (!fread(moremagic, 8, 1, fp)) {
+		Mix_SetError("Couldn't read from '%s'", file);
+		return(NULL);
+	}
 	magic[4] = '\0';
+	moremagic[8] = '\0';
 	fclose(fp);
 
 	/* Figure out the file extension, so we can determine the type */
@@ -422,7 +536,7 @@ Mix_Music *Mix_LoadMUS(const char *file)
 	   AIFF files have the magic 12 bytes "FORM" XXXX "AIFF"
 	 */
 	if ( (ext && MIX_string_equals(ext, "WAV")) ||
-	     (strcmp((char *)magic, "RIFF") == 0) ||
+	     ((strcmp((char *)magic, "RIFF") == 0) && (strcmp((char *)(moremagic+4), "WAVE") == 0)) ||
 	     (strcmp((char *)magic, "FORM") == 0) ) {
 		music->type = MUS_WAV;
 		music->data.wave = WAVStream_LoadSong(file, (char *)magic);
@@ -436,7 +550,9 @@ Mix_Music *Mix_LoadMUS(const char *file)
 	/* MIDI files have the magic four bytes "MThd" */
 	if ( (ext && MIX_string_equals(ext, "MID")) ||
 	     (ext && MIX_string_equals(ext, "MIDI")) ||
-	     strcmp((char *)magic, "MThd") == 0 ) {
+	     strcmp((char *)magic, "MThd") == 0  ||
+	     ( strcmp((char *)magic, "RIFF") == 0  &&
+	  	strcmp((char *)(moremagic+4), "RMID") == 0 ) ) {
 		music->type = MUS_MID;
 #ifdef USE_NATIVE_MIDI
   		if ( native_midi_ok ) {
@@ -476,19 +592,23 @@ Mix_Music *Mix_LoadMUS(const char *file)
 	if ( (ext && MIX_string_equals(ext, "MPG")) ||
 	     (ext && MIX_string_equals(ext, "MP3")) ||
 	     (ext && MIX_string_equals(ext, "MPEG")) ||
-	     magic[0]==0xFF && (magic[1]&0xF0)==0xF0) {
-		SMPEG_Info info;
-		music->type = MUS_MP3;
-		music->data.mp3 = SMPEG_new(file, &info, 0);
-		if(!info.has_audio){
-			Mix_SetError("MPEG file does not have any audio stream.");
+	     (magic[0] == 0xFF && (magic[1] & 0xF0) == 0xF0) ) {
+		if ( Mix_InitMP3() == 0 ) {
+			SMPEG_Info info;
+			music->type = MUS_MP3;
+			music->data.mp3 = smpeg.SMPEG_new(file, &info, 0);
+			if ( !info.has_audio ) {
+				Mix_SetError("MPEG file does not have any audio stream.");
+				music->error = 1;
+			} else {
+				smpeg.SMPEG_actualSpec(music->data.mp3, &used_mixer);
+			}
+		} else {
 			music->error = 1;
-		}else{
-			SMPEG_actualSpec(music->data.mp3, &used_mixer);
 		}
 	} else
 #endif
-#ifdef MOD_MUSIC
+#if defined(MOD_MUSIC) || defined(LIBMIKMOD_MUSIC)
 	if ( 1 ) {
 		music->type = MUS_MOD;
 		music->data.module = MikMod_LoadSong((char *)file, 64);
@@ -500,7 +620,7 @@ Mix_Music *Mix_LoadMUS(const char *file)
 			music->data.module->extspd  = 1;
 			music->data.module->panflag = 1;
 			music->data.module->wrap    = 0;
-			music->data.module->loop    = 0;
+			music->data.module->loop    = 1;
 #if 0 /* Don't set fade out by default - unfortunately there's no real way
          to query the status of the song or set trigger actions.  Hum. */
 			music->data.module->fadeout = 1;
@@ -548,7 +668,7 @@ void Mix_FreeMusic(Mix_Music *music)
 				WAVStream_FreeSong(music->data.wave);
 				break;
 #endif
-#ifdef MOD_MUSIC
+#if defined(MOD_MUSIC) || defined(LIBMIKMOD_MUSIC)
 			case MUS_MOD:
 				MikMod_FreeSong(music->data.module);
 				break;
@@ -574,7 +694,8 @@ void Mix_FreeMusic(Mix_Music *music)
 #endif
 #ifdef MP3_MUSIC
 			case MUS_MP3:
-				SMPEG_delete(music->data.mp3);
+				smpeg.SMPEG_delete(music->data.mp3);
+				Mix_QuitMP3();
 				break;
 #endif
 			default:
@@ -617,10 +738,8 @@ static int music_internal_play(Mix_Music *music, double position)
 	music_playing = music;
 
 	/* Set the initial volume */
-	if ( music->fading == MIX_FADING_IN ) {
-		music_internal_volume(0);
-	} else {
-		music_internal_volume(music_volume);
+	if ( music->type != MUS_MOD ) {
+		music_internal_initialize_volume();
 	}
 
 	/* Set up for playback */
@@ -635,9 +754,11 @@ static int music_internal_play(Mix_Music *music, double position)
 		WAVStream_Start(music->data.wave);
 		break;
 #endif
-#ifdef MOD_MUSIC
+#if defined(MOD_MUSIC) || defined(LIBMIKMOD_MUSIC)
 	    case MUS_MOD:
 		Player_Start(music->data.module);
+		/* Player_SetVolume() does nothing before Player_Start() */
+		music_internal_initialize_volume();
 		break;
 #endif
 #ifdef MID_MUSIC
@@ -661,9 +782,9 @@ static int music_internal_play(Mix_Music *music, double position)
 #endif
 #ifdef MP3_MUSIC
 	    case MUS_MP3:
-		SMPEG_enableaudio(music->data.mp3,1);
-		SMPEG_enablevideo(music->data.mp3,0);
-		SMPEG_play(music_playing->data.mp3);
+		smpeg.SMPEG_enableaudio(music->data.mp3,1);
+		smpeg.SMPEG_enablevideo(music->data.mp3,0);
+		smpeg.SMPEG_play(music_playing->data.mp3);
 		break;
 #endif
 	    default:
@@ -739,7 +860,7 @@ int music_internal_position(double position)
 	int retval = 0;
 
 	switch (music_playing->type) {
-#ifdef MOD_MUSIC
+#if defined(MOD_MUSIC) || defined(LIBMIKMOD_MUSIC)
 	    case MUS_MOD:
 		Player_SetPosition((UWORD)position);
 		break;
@@ -752,10 +873,10 @@ int music_internal_position(double position)
 #ifdef MP3_MUSIC
 	    case MUS_MP3:
 		if ( position > 0.0 ) {
-			SMPEG_skip(music_playing->data.mp3, position);
+			smpeg.SMPEG_skip(music_playing->data.mp3, (float)position);
 		} else {
-			SMPEG_rewind(music_playing->data.mp3);
-			SMPEG_play(music_playing->data.mp3);
+			smpeg.SMPEG_rewind(music_playing->data.mp3);
+			smpeg.SMPEG_play(music_playing->data.mp3);
 		}
 		break;
 #endif
@@ -785,6 +906,16 @@ int Mix_SetMusicPosition(double position)
 	return(retval);
 }
 
+/* Set the music's initial volume */
+static void music_internal_initialize_volume(void)
+{
+	if ( music_playing->fading == MIX_FADING_IN ) {
+		music_internal_volume(0);
+	} else {
+		music_internal_volume(music_volume);
+	}
+}
+
 /* Set the music volume */
 static void music_internal_volume(int volume)
 {
@@ -799,7 +930,7 @@ static void music_internal_volume(int volume)
 		WAVStream_SetVolume(volume);
 		break;
 #endif
-#ifdef MOD_MUSIC
+#if defined(MOD_MUSIC) || defined(LIBMIKMOD_MUSIC)
 	    case MUS_MOD:
 		Player_SetVolume((SWORD)volume);
 		break;
@@ -825,7 +956,7 @@ static void music_internal_volume(int volume)
 #endif
 #ifdef MP3_MUSIC
 	    case MUS_MP3:
-		SMPEG_setvolume(music_playing->data.mp3,(int)(((float)volume/(float)MIX_MAX_VOLUME)*100.0));
+		smpeg.SMPEG_setvolume(music_playing->data.mp3,(int)(((float)volume/(float)MIX_MAX_VOLUME)*100.0));
 		break;
 #endif
 	    default:
@@ -867,7 +998,7 @@ static void music_internal_halt(void)
 		WAVStream_Stop();
 		break;
 #endif
-#ifdef MOD_MUSIC
+#if defined(MOD_MUSIC) || defined(LIBMIKMOD_MUSIC)
 	    case MUS_MOD:
 		Player_Stop();
 		break;
@@ -893,7 +1024,7 @@ static void music_internal_halt(void)
 #endif
 #ifdef MP3_MUSIC
 	    case MUS_MP3:
-		SMPEG_stop(music_playing->data.mp3);
+		smpeg.SMPEG_stop(music_playing->data.mp3);
 		break;
 #endif
 	    default:
@@ -919,11 +1050,30 @@ int Mix_FadeOutMusic(int ms)
 {
 	int retval = 0;
 
+	if (ms <= 0) {  /* just halt immediately. */
+		Mix_HaltMusic();
+		return 1;
+	}
+
 	SDL_LockAudio();
-	if ( music_playing && (music_playing->fading == MIX_NO_FADING) ) {
+	if ( music_playing) {
+                int fade_steps = (ms + ms_per_step - 1)/ms_per_step;
+                if ( music_playing->fading == MIX_NO_FADING ) {
+	        	music_playing->fade_step = 0;
+                } else {
+                        int step;
+                        int old_fade_steps = music_playing->fade_steps;
+                        if ( music_playing->fading == MIX_FADING_OUT ) {
+                                step = music_playing->fade_step;
+                        } else {
+                                step = old_fade_steps
+                                        - music_playing->fade_step + 1;
+                        }
+                        music_playing->fade_step = (step * fade_steps)
+                                / old_fade_steps;
+                }
 		music_playing->fading = MIX_FADING_OUT;
-		music_playing->fade_step = 0;
-		music_playing->fade_steps = ms/ms_per_step;
+		music_playing->fade_steps = fade_steps;
 		retval = 1;
 	}
 	SDL_UnlockAudio();
@@ -984,7 +1134,7 @@ static int music_internal_playing()
 		}
 		break;
 #endif
-#ifdef MOD_MUSIC
+#if defined(MOD_MUSIC) || defined(LIBMIKMOD_MUSIC)
 	    case MUS_MOD:
 		if ( ! Player_Active() ) {
 			playing = 0;
@@ -1016,7 +1166,7 @@ static int music_internal_playing()
 #endif
 #ifdef MP3_MUSIC
 	    case MUS_MP3:
-		if ( SMPEG_status(music_playing->data.mp3) != SMPEG_PLAYING )
+		if ( smpeg.SMPEG_status(music_playing->data.mp3) != SMPEG_PLAYING )
 			playing = 0;
 		break;
 #endif
@@ -1057,11 +1207,26 @@ int Mix_SetMusicCMD(const char *command)
 	return(0);
 }
 
+#ifdef LIBMIKMOD_MUSIC
+static int _pl_synchro_value;
+void Player_SetSynchroValue(int i)
+{
+	fprintf(stderr,"SDL_mixer: Player_SetSynchroValue is not supported.\n");
+	_pl_synchro_value = i;
+}
+
+int Player_GetSynchroValue(void)
+{
+	fprintf(stderr,"SDL_mixer: Player_GetSynchroValue is not supported.\n");
+	return _pl_synchro_value;
+}
+#endif
+
 int Mix_SetSynchroValue(int i)
 {
 	if ( music_playing && ! music_stopped ) {
 		switch (music_playing->type) {
-#ifdef MOD_MUSIC
+#if defined(MOD_MUSIC) || defined(LIBMIKMOD_MUSIC)
 		    case MUS_MOD:
 			if ( ! Player_Active() ) {
 				return(-1);
@@ -1083,7 +1248,7 @@ int Mix_GetSynchroValue(void)
 {
 	if ( music_playing && ! music_stopped ) {
 		switch (music_playing->type) {
-#ifdef MOD_MUSIC
+#if defined(MOD_MUSIC) || defined(LIBMIKMOD_MUSIC)
 		    case MUS_MOD:
 			if ( ! Player_Active() ) {
 				return(-1);
@@ -1108,10 +1273,191 @@ void close_music(void)
 #ifdef CMD_MUSIC
 	Mix_SetMusicCMD(NULL);
 #endif
-#ifdef MOD_MUSIC
+#if defined(MOD_MUSIC) || defined(LIBMIKMOD_MUSIC)
 	MikMod_Exit();
+# ifndef LIBMIKMOD_MUSIC
 	MikMod_UnregisterAllLoaders();
 	MikMod_UnregisterAllDrivers();
+# endif
+#endif
+#ifdef MID_MUSIC
+# ifdef USE_TIMIDITY_MIDI
+	Timidity_Close();
+# endif
 #endif
 }
 
+# ifdef LIBMIKMOD_MUSIC
+typedef struct
+{
+	MREADER mr;
+	int offset;
+	int eof;
+	SDL_RWops *rw;
+} LMM_MREADER;
+BOOL LMM_Seek(struct MREADER *mr,long to,int dir)
+{
+	int at;
+	LMM_MREADER* lmmmr=(LMM_MREADER*)mr;
+	if(dir==SEEK_SET)
+		to+=lmmmr->offset;
+	at=SDL_RWseek(lmmmr->rw, to, dir);
+	return at<lmmmr->offset;
+}
+long LMM_Tell(struct MREADER *mr)
+{
+	int at;
+	LMM_MREADER* lmmmr=(LMM_MREADER*)mr;
+	at=SDL_RWtell(lmmmr->rw)-lmmmr->offset;
+	return at;
+}
+BOOL LMM_Read(struct MREADER *mr,void *buf,size_t sz)
+{
+	int got;
+	LMM_MREADER* lmmmr=(LMM_MREADER*)mr;
+	got=SDL_RWread(lmmmr->rw, buf, sz, 1);
+	return got;
+}
+int LMM_Get(struct MREADER *mr)
+{
+	unsigned char c;
+	int i=EOF;
+	LMM_MREADER* lmmmr=(LMM_MREADER*)mr;
+	if(SDL_RWread(lmmmr->rw,&c,1,1))
+		i=c;
+	return i;
+}
+BOOL LMM_Eof(struct MREADER *mr)
+{
+	int offset;
+	LMM_MREADER* lmmmr=(LMM_MREADER*)mr;
+	offset=LMM_Tell(mr);
+	return offset>=lmmmr->eof;
+}
+MODULE *MikMod_LoadSongRW(SDL_RWops *rw, int maxchan)
+{
+	LMM_MREADER lmmmr={
+		LMM_Seek,
+		LMM_Tell,
+		LMM_Read,
+		LMM_Get,
+		LMM_Eof,
+		0,
+		0,
+		rw
+	};
+	MODULE *m;
+	lmmmr.offset=SDL_RWtell(rw);
+	SDL_RWseek(rw,0,SEEK_END);
+	lmmmr.eof=SDL_RWtell(rw);
+	SDL_RWseek(rw,lmmmr.offset,SEEK_SET);
+	m=Player_LoadGeneric((MREADER*)&lmmmr,maxchan,0);
+	return m;
+}
+# endif
+
+Mix_Music *Mix_LoadMUS_RW(SDL_RWops *rw) {
+	Uint8 magic[5];	  /*Apparently there is no way to check if the file is really a MOD,*/
+	/*		    or there are too many formats supported by MikMod or MikMod does */
+	/*		    this check by itself. If someone implements other formats (e.g. MP3) */
+	/*		    the check can be uncommented */
+	Mix_Music *music;
+	int start;
+
+	/* Figure out what kind of file this is */
+	start = SDL_RWtell(rw);
+	if (SDL_RWread(rw,magic,1,4)!=4) {
+		Mix_SetError("Couldn't read from RWops");
+		return NULL;
+	}
+	SDL_RWseek(rw, start, SEEK_SET);
+	magic[4]='\0';
+
+	/* Allocate memory for the music structure */
+	music=(Mix_Music *)malloc(sizeof(Mix_Music));
+	if (music==NULL ) {
+		Mix_SetError("Out of memory");
+		return(NULL);
+	}
+	music->error = 0;
+
+#ifdef OGG_MUSIC
+	/* Ogg Vorbis files have the magic four bytes "OggS" */
+	if ( strcmp((char *)magic, "OggS") == 0 ) {
+		music->type = MUS_OGG;
+		music->data.ogg = OGG_new_RW(rw);
+		if ( music->data.ogg == NULL ) {
+			music->error = 1;
+		}
+	} else
+#endif
+#ifdef MP3_MUSIC
+	if ( magic[0] == 0xFF && (magic[1] & 0xF0) == 0xF0 ) {
+		if ( Mix_InitMP3() == 0 ) {
+			SMPEG_Info info;
+			music->type = MUS_MP3;
+			music->data.mp3 = smpeg.SMPEG_new_rwops(rw, &info, 0);
+			if ( !info.has_audio ) {
+				Mix_SetError("MPEG file does not have any audio stream.");
+				music->error = 1;
+			} else {
+				smpeg.SMPEG_actualSpec(music->data.mp3, &used_mixer);
+			}
+		} else {
+			music->error = 1;
+		}
+	} else
+#endif
+#ifdef MID_MUSIC
+	/* MIDI files have the magic four bytes "MThd" */
+	if ( strcmp((char *)magic, "MThd") == 0 ) {
+		music->type = MUS_MID;
+		music->error = 1;
+#ifdef USE_NATIVE_MIDI
+		if ( native_midi_ok ) {
+			music->data.nativemidi = native_midi_loadsong_RW(rw);
+			if ( music->data.nativemidi ) {
+				music->error = 0;
+			}
+		} MIDI_ELSE
+#endif
+#ifdef USE_TIMIDITY_MIDI
+		if ( timidity_ok ) {
+			music->data.midi = Timidity_LoadSong_RW(rw);
+			if ( music->data.midi ) {
+				music->error = 0;
+			}
+		}
+#endif
+	} else
+#endif
+#if defined(MOD_MUSIC) || defined(LIBMIKMOD_MUSIC)
+	if (1) {
+		music->type=MUS_MOD;
+		music->data.module=MikMod_LoadSongRW(rw,64);
+		if (music->data.module==NULL) {
+			Mix_SetError("%s",MikMod_strerror(MikMod_errno));
+			music->error=1;
+		} else {
+			/* Stop implicit looping, fade out and other flags. */
+			music->data.module->extspd  = 1;
+			music->data.module->panflag = 1;
+			music->data.module->wrap    = 0;
+			music->data.module->loop    = 0;
+#if 0 /* Don't set fade out by default - unfortunately there's no real way
+         to query the status of the song or set trigger actions.  Hum. */
+			music->data.module->fadeout = 1;
+#endif
+		}
+	} else
+#endif
+	{
+		Mix_SetError("Unrecognized music format");
+		music->error=1;
+	}
+	if (music->error) {
+		free(music);
+		music=NULL;
+	}
+	return(music);
+}
